@@ -66,9 +66,6 @@
 #ifdef HAVE_SYS_DKIO_H
 #include <sys/dkio.h>
 #endif
-#ifdef HAVE_SYS_STAT_H
-#include <sys/stat.h>
-#endif
 /* linux/fs.h may not be needed because sys/mount.h is present */
 /* #ifdef HAVE_LINUX_FS_H */
 /* #include <linux/fs.h> */
@@ -109,6 +106,7 @@
 #include "log.h"
 #include "hdaccess.h"
 #include "alignio.h"
+#include "hpa_dco.h"
 
 extern const arch_fnct_t arch_gpt;
 extern const arch_fnct_t arch_mac;
@@ -124,13 +122,13 @@ static void autoset_geometry(disk_t * disk_car, const unsigned char *buffer, con
 static const char *file_description(disk_t *disk_car);
 static const char *file_description_short(disk_t *disk_car);
 static int file_clean(disk_t *disk_car);
-static int file_read(disk_t *disk_car, const unsigned int count, void *buf, const uint64_t offset);
-static int file_write(disk_t *disk_car, const unsigned int count, const void *buf, const uint64_t offset);
-static int file_nowrite(disk_t *disk_car, const unsigned int count, const void *buf, const uint64_t offset);
+static int file_pread(disk_t *disk_car, void *buf, const unsigned int count, const uint64_t offset);
+static int file_pwrite(disk_t *disk_car, const void *buf, const unsigned int count, const uint64_t offset);
+static int file_nopwrite(disk_t *disk_car, const void *buf, const unsigned int count, const uint64_t offset);
 static int file_sync(disk_t *disk_car);
 #ifndef DJGPP
 static uint64_t compute_device_size(const int hd_h, const char *device, const int verbose, const unsigned int sector_size);
-static void disk_get_model(const int hd_h, disk_t *disk_car, const int verbose);
+static void disk_get_model(const int hd_h, disk_t *disk, const unsigned int verbose);
 #endif
 
 int generic_clean(disk_t *disk_car)
@@ -198,7 +196,7 @@ static list_disk_t *hd_glob_parse(const char *device_pattern, list_disk_t *list_
 
 list_disk_t *hd_parse(list_disk_t *list_disk, const int verbose, const arch_fnct_t *arch, const int testdisk_mode)
 {
-  int i;
+  unsigned int i;
 #ifdef DJGPP
   int ind_stop=0;
   for(i=0x80;(i<0x88)&&!ind_stop;i++)
@@ -532,23 +530,24 @@ static unsigned int disk_get_sector_size(const int hd_h, const char *device, con
   return DEFAULT_SECTOR_SIZE;
 }
 
-static void disk_get_geometry(CHS_t *CHS, const int hd_h, const char *device, const int verbose)
+static void disk_get_geometry(CHSgeometry_t *geom, const int hd_h, const char *device, const int verbose)
 {
   if(verbose>1)
     log_verbose("disk_get_geometry for %s\n", device);
 #ifdef HDIO_GETGEO_BIG
-  if(CHS->sector==0)
+  if(geom->sectors_per_head==0)
   {
-    struct hd_big_geometry geometry_big;
-    if (ioctl(hd_h, HDIO_GETGEO_BIG, &geometry_big)>=0)
+    struct hd_big_geometry geometry;
+    if (ioctl(hd_h, HDIO_GETGEO_BIG, &geometry)>=0)
     { /* I can get the geometry */
       if(verbose>1)
       {
-        log_verbose("disk_get_geometry HDIO_GETGEO_BIG %s Ok (%u,%u,%u)\n",device,geometry_big.cylinders,geometry_big.heads,geometry_big.sectors);
+        log_verbose("disk_get_geometry HDIO_GETGEO_BIG %s Ok (%u,%u,%u)\n",
+	    device, geometry.cylinders, geometry.heads, geometry.sectors);
       }
-      CHS->cylinder= geometry_big.cylinders-1;
-      CHS->head=geometry_big.heads-1;
-      CHS->sector= geometry_big.sectors;
+      geom->cylinders=geometry.cylinders;
+      geom->heads_per_cylinder=geometry.heads;
+      geom->sectors_per_head=geometry.sectors;
     }
     else
     {
@@ -560,29 +559,30 @@ static void disk_get_geometry(CHS_t *CHS, const int hd_h, const char *device, co
   }
 #endif
 #ifdef HDIO_GETGEO
-  if(CHS->sector==0)
+  if(geom->sectors_per_head==0)
   {
     struct hd_geometry geometry;
     if(ioctl(hd_h, HDIO_GETGEO, &geometry)>=0)
     { /* I can get the geometry */
       if(verbose>1)
       {
-	log_verbose("disk_get_geometry HDIO_GETGEO %s Ok (%u,%u,%u)\n",device,geometry.cylinders,geometry.heads,geometry.sectors);
+	log_verbose("disk_get_geometry HDIO_GETGEO %s Ok (%u,%u,%u)\n",
+	    device, geometry.cylinders, geometry.heads, geometry.sectors);
       }
-      CHS->cylinder= geometry.cylinders-1;
-      CHS->head=geometry.heads-1;
-      CHS->sector= geometry.sectors;
+      geom->cylinders=geometry.cylinders;
+      geom->heads_per_cylinder=geometry.heads;
+      geom->sectors_per_head=geometry.sectors;
     }
   }
 #endif
 #ifdef DKIOCGGEOM
-  if(CHS->sector==0)
+  if(geom->sectors_per_head==0)
   {
     struct dk_geom dkgeom;
     if (ioctl (hd_h, DKIOCGGEOM, &dkgeom)>=0) {
-      CHS->cylinder= dkgeom.dkg_ncyl-1;
-      CHS->head=dkgeom.dkg_nhead-1;
-      CHS->sector=dkgeom.dkg_nsect;
+      geom->cylinders= dkgeom.dkg_ncyl;
+      geom->heads_per_cylinder=dkgeom.dkg_nhead;
+      geom->sectors_per_head=dkgeom.dkg_nsect;
       if(verbose>1)
       {
 	log_verbose("disk_get_geometry DKIOCGGEOM %s Ok\n",device);
@@ -591,7 +591,7 @@ static void disk_get_geometry(CHS_t *CHS, const int hd_h, const char *device, co
   }
 #endif
 #ifdef DIOCGDINFO
-  if(CHS->sector==0)
+  if(geom->sectors_per_head==0)
   {
     struct disklabel geometry;
     if (ioctl(hd_h, DIOCGDINFO, &geometry)==0)
@@ -600,9 +600,9 @@ static void disk_get_geometry(CHS_t *CHS, const int hd_h, const char *device, co
       {
 	log_verbose("disk_get_geometry DIOCGDINFO %s Ok\n",device);
       }
-      CHS->cylinder=geometry.d_ncylinders-1;
-      CHS->head=geometry.d_ntracks-1;
-      CHS->sector=geometry.d_nsectors;
+      geom->cylinders=geometry.d_ncylinders;
+      geom->heads_per_cylinder=geometry.d_ntracks;
+      geom->sectors_per_head=geometry.d_nsectors;
     }
     else
     {
@@ -614,7 +614,7 @@ static void disk_get_geometry(CHS_t *CHS, const int hd_h, const char *device, co
   }
 #endif
 #ifdef DIOCGFWSECTORS
-  if(CHS->sector==0)
+  if(geom->sectors_per_head==0)
   {
     int error;
     unsigned int u;
@@ -635,9 +635,9 @@ static void disk_get_geometry(CHS_t *CHS, const int hd_h, const char *device, co
 	{
 	  log_verbose("disk_get_geometry DIOCGFWHEADS %s Ok\n",device);
 	}
-	CHS->cylinder=0;
-	CHS->head=heads-1;
-	CHS->sector=sectors;
+	geom->cylinders=0;
+	geom->heads_per_cylinder=heads;
+	geom->sectors_per_head=sectors;
       }
     }
   }
@@ -650,14 +650,14 @@ static void disk_get_geometry(CHS_t *CHS, const int hd_h, const char *device, co
 #else
     handle=(HANDLE)_get_osfhandle(hd_h);
 #endif
-    return disk_get_geometry_win32(CHS, handle, device, verbose);
+    return disk_get_geometry_win32(geom, handle, device, verbose);
   }
 #endif
-  if(CHS->sector!=0)
+  if(geom->sectors_per_head>0 && geom->heads_per_cylinder>0)
     return ;
-  CHS->cylinder=0;
-  CHS->head=0;
-  CHS->sector=1;
+  geom->cylinders=0;
+  geom->heads_per_cylinder=1;
+  geom->sectors_per_head=1;
   if(verbose>1)
   {
     log_error("disk_get_geometry default geometry for %s\n", device);
@@ -747,28 +747,29 @@ void update_disk_car_fields(disk_t *disk_car)
 {
   if(disk_car->disk_real_size==0)
   {
-    if(disk_car->CHS.cylinder>0)
+    if(disk_car->geom.cylinders>0)
     {
       log_warning("Fix disk size using CHS\n");
-      disk_car->disk_real_size=(uint64_t)(disk_car->CHS.cylinder+1)*(disk_car->CHS.head+1)*disk_car->CHS.sector*disk_car->sector_size;
+      disk_car->disk_real_size=(uint64_t)disk_car->geom.cylinders * disk_car->geom.heads_per_cylinder *
+	disk_car->geom.sectors_per_head * disk_car->sector_size;
     }
   }
   else
   {
-    unsigned int cylinder_num;
-    cylinder_num=disk_car->disk_real_size / ((uint64_t)disk_car->CHS.head+1) / (uint64_t)disk_car->CHS.sector/(uint64_t)disk_car->sector_size;
-    if(cylinder_num>0 && disk_car->CHS.cylinder+1!= cylinder_num)
+    const unsigned long int cylinder_num=disk_car->disk_real_size /
+      (uint64_t)disk_car->geom.heads_per_cylinder /
+      (uint64_t)disk_car->geom.sectors_per_head /
+      (uint64_t)disk_car->sector_size;
+    if(cylinder_num>0 && disk_car->geom.cylinders != cylinder_num)
     {
-      log_debug("Fix cylinder count for %s: number of cylinders %u !=  %u (calculated)\n",
-	  disk_car->device, disk_car->CHS.cylinder+1, cylinder_num);
-      disk_car->CHS.cylinder= cylinder_num-1;
+      log_debug("Fix cylinder count for %s: number of cylinders %lu !=  %lu (calculated)\n",
+	  disk_car->device, disk_car->geom.cylinders, cylinder_num);
+      disk_car->geom.cylinders = cylinder_num;
     }
   }
+  if(disk_car->geom.cylinders == 0)
+    disk_car->geom.cylinders++;
   disk_car->disk_size=disk_car->disk_real_size;
-  /*
-  disk_car->disk_size=td_max(disk_car->disk_real_size,
-      (uint64_t)(disk_car->CHS.cylinder+1)*(disk_car->CHS.head+1)*disk_car->CHS.sector*disk_car->sector_size);
-  */
 }
 
 #ifdef TARGET_LINUX
@@ -906,7 +907,7 @@ static int scsi_query_product_info (const int hd_h, char **vendor, char **produc
 #endif
 
 #ifndef DJGPP
-static void disk_get_model(const int hd_h, disk_t *dev, const int verbose)
+static void disk_get_model(const int hd_h, disk_t *dev, const unsigned int verbose)
 {
 #ifdef TARGET_LINUX
   if(dev->model!=NULL)
@@ -931,6 +932,7 @@ static void disk_get_model(const int hd_h, disk_t *dev, const int verbose)
     return;
   {
     struct hd_driveid       hdi;
+    memset(&hdi, 0, sizeof(hdi));
     if (ioctl (hd_h, HDIO_GET_IDENTITY, &hdi)==0)
     {
       char hdi_buf[41];
@@ -1021,9 +1023,9 @@ static const char *file_description(disk_t *disk_car)
 {
   const struct info_file_struct *data=(const struct info_file_struct *)disk_car->data;
   char buffer_disk_size[100];
-  snprintf(disk_car->description_txt, sizeof(disk_car->description_txt),"Disk %s - %s - CHS %u %u %u%s",
+  snprintf(disk_car->description_txt, sizeof(disk_car->description_txt),"Disk %s - %s - CHS %lu %u %u%s",
       disk_car->device, size_to_unit(disk_car->disk_size,buffer_disk_size),
-      disk_car->CHS.cylinder+1, disk_car->CHS.head+1, disk_car->CHS.sector,
+      disk_car->geom.cylinders, disk_car->geom.heads_per_cylinder, disk_car->geom.sectors_per_head,
       ((data->mode&O_RDWR)==O_RDWR?"":" (RO)"));
   return disk_car->description_txt;
 }
@@ -1063,7 +1065,7 @@ static int file_clean(disk_t *disk_car)
   return generic_clean(disk_car);
 }
 
-static int file_read_aux(disk_t *disk_car, void *buf, const unsigned int count, const uint64_t offset)
+static int file_pread_aux(disk_t *disk_car, void *buf, const unsigned int count, const uint64_t offset)
 {
   long int ret=-1;
   int fd=((struct info_file_struct *)disk_car->data)->handle;
@@ -1075,7 +1077,7 @@ static int file_read_aux(disk_t *disk_car, void *buf, const unsigned int count,
 #ifdef __MINGW32__
     if(_lseeki64(fd,offset,SEEK_SET)==-1)
     {
-      log_error("file_read(%d,%u,buffer,%lu(%u/%u/%u)) seek err %s\n", fd,
+      log_error("file_pread(%d,%u,buffer,%lu(%u/%u/%u)) seek err %s\n", fd,
           (unsigned)(count/disk_car->sector_size), (long unsigned int)(offset/disk_car->sector_size),
           offset2cylinder(disk_car,offset), offset2head(disk_car,offset), offset2sector(disk_car,offset),
           strerror(errno));
@@ -1084,7 +1086,7 @@ static int file_read_aux(disk_t *disk_car, void *buf, const unsigned int count,
 #else
     if(lseek(fd,offset,SEEK_SET)==(off_t)-1)
     {
-      log_error("file_read(%d,%u,buffer,%lu(%u/%u/%u)) lseek err %s\n", fd,
+      log_error("file_pread(%d,%u,buffer,%lu(%u/%u/%u)) lseek err %s\n", fd,
           (unsigned)(count/disk_car->sector_size), (long unsigned int)(offset/disk_car->sector_size),
           offset2cylinder(disk_car,offset), offset2head(disk_car,offset), offset2sector(disk_car,offset),
           strerror(errno));
@@ -1111,7 +1113,7 @@ static int file_read_aux(disk_t *disk_car, void *buf, const unsigned int count,
   {
     if(offset+count <= disk_car->disk_size && offset+count <= disk_car->disk_real_size)
     {
-      log_error("file_read(%d,%u,buffer,%lu(%u/%u/%u)) read err: ", fd,
+      log_error("file_pread(%d,%u,buffer,%lu(%u/%u/%u)) read err: ", fd,
           (unsigned)(count/disk_car->sector_size), (long unsigned)(offset/disk_car->sector_size),
           offset2cylinder(disk_car,offset), offset2head(disk_car,offset), offset2sector(disk_car,offset));
       if(ret<0)
@@ -1122,18 +1124,21 @@ static int file_read_aux(disk_t *disk_car, void *buf, const unsigned int count,
         log_error("Partial read\n");
     }
     if(ret<=0)
+    {
+      memset(buf, 0, count);
       return -1;
+    }
     memset((char*)buf+ret,0,count-ret);
   }
-  return 0;
+  return ret;
 }
 
-static int file_read(disk_t *disk_car,const unsigned int count, void *buf, const uint64_t offset)
+static int file_pread(disk_t *disk_car, void *buf, const unsigned int count, const uint64_t offset)
 {
-  return align_read(&file_read_aux, disk_car, buf, count, offset);
+  return align_pread(&file_pread_aux, disk_car, buf, count, offset);
 }
 
-static int file_write_aux(disk_t *disk_car, const void *buf, const unsigned int count, const uint64_t offset)
+static int file_pwrite_aux(disk_t *disk_car, const void *buf, const unsigned int count, const uint64_t offset)
 {
   int fd=((struct info_file_struct *)disk_car->data)->handle;
   long int ret=-1;
@@ -1145,7 +1150,7 @@ static int file_write_aux(disk_t *disk_car, const void *buf, const unsigned int
 #ifdef __MINGW32__
     if(_lseeki64(fd,offset,SEEK_SET)==-1)
     {
-      log_error("file_write(%d,%u,buffer,%lu(%u/%u/%u)) seek err %s\n", fd,
+      log_error("file_pwrite(%d,%u,buffer,%lu(%u/%u/%u)) seek err %s\n", fd,
           (unsigned)(count/disk_car->sector_size),
           (long unsigned)(offset/disk_car->sector_size),
           offset2cylinder(disk_car,offset),offset2head(disk_car,offset),offset2sector(disk_car,offset),strerror(errno));
@@ -1154,7 +1159,7 @@ static int file_write_aux(disk_t *disk_car, const void *buf, const unsigned int
 #else
     if(lseek(fd,offset,SEEK_SET)==-1)
     {
-      log_error("file_write(%d,%u,buffer,%lu(%u/%u/%u)) seek err %s\n", fd,(unsigned)(count/disk_car->sector_size),(long unsigned)(offset/disk_car->sector_size),
+      log_error("file_pwrite(%d,%u,buffer,%lu(%u/%u/%u)) seek err %s\n", fd,(unsigned)(count/disk_car->sector_size),(long unsigned)(offset/disk_car->sector_size),
           offset2cylinder(disk_car,offset),offset2head(disk_car,offset),offset2sector(disk_car,offset),strerror(errno));
       return -1;
     }
@@ -1164,22 +1169,22 @@ static int file_write_aux(disk_t *disk_car, const void *buf, const unsigned int
   disk_car->write_used=1;
   if(ret!=count)
   {
-    log_error("file_write(%d,%u,buffer,%lu(%u/%u/%u)) write err %s\n", fd,(unsigned)(count/disk_car->sector_size),(long unsigned)(offset/disk_car->sector_size),
+    log_error("file_pwrite(%d,%u,buffer,%lu(%u/%u/%u)) write err %s\n", fd,(unsigned)(count/disk_car->sector_size),(long unsigned)(offset/disk_car->sector_size),
         offset2cylinder(disk_car,offset),offset2head(disk_car,offset),offset2sector(disk_car,offset),(ret<0?strerror(errno):"File truncated"));
     return -1;
   }
-  return 0;
+  return ret;
 }
 
-static int file_write(disk_t *disk_car, const unsigned int count, const void *buf, const uint64_t offset)
+static int file_pwrite(disk_t *disk_car, const void *buf, const unsigned int count, const uint64_t offset)
 {
-  return align_write(&file_read_aux, &file_write_aux, disk_car, buf, count, offset);
+  return align_pwrite(&file_pread_aux, &file_pwrite_aux, disk_car, buf, count, offset);
 }
 
-static int file_nowrite(disk_t *disk_car,const unsigned int count, const void *buf, const uint64_t offset)
+static int file_nopwrite(disk_t *disk_car, const void *buf, const unsigned int count, const uint64_t offset)
 {
   struct info_file_struct *data=(struct info_file_struct *)disk_car->data;
-  log_warning("file_nowrite(%d,%u,buffer,%lu(%u/%u/%u)) write refused\n", data->handle,
+  log_warning("file_nopwrite(%d,%u,buffer,%lu(%u/%u/%u)) write refused\n", data->handle,
       (unsigned)(count/disk_car->sector_size),(long unsigned)(offset/disk_car->sector_size),
       offset2cylinder(disk_car,offset),offset2head(disk_car,offset),offset2sector(disk_car,offset));
   return -1;
@@ -1200,29 +1205,27 @@ static void autoset_geometry(disk_t * disk_car, const unsigned char *buffer, con
 {
   if(disk_car->arch->get_geometry_from_mbr!=NULL)
   {
-    CHS_t geometry;
-    geometry.cylinder=0;
-    geometry.head=0;
-    geometry.sector=0;
+    CHSgeometry_t geometry;
+    geometry.cylinders=0;
+    geometry.heads_per_cylinder=0;
+    geometry.sectors_per_head=0;
     disk_car->arch->get_geometry_from_mbr(buffer, verbose, &geometry);
     disk_car->autodetect=1;
-    if(geometry.sector>0)
+    if(geometry.sectors_per_head > 0)
     {
-      disk_car->CHS.head=geometry.head;
-      disk_car->CHS.sector=geometry.sector;
+      disk_car->geom.heads_per_cylinder=geometry.heads_per_cylinder;
+      disk_car->geom.sectors_per_head=geometry.sectors_per_head;
     }
     else
     {
-      disk_car->CHS.head=255-1;
-      disk_car->CHS.sector=63;
+      disk_car->geom.heads_per_cylinder=255;
+      disk_car->geom.sectors_per_head=63;
     }
   }
   /* Round up because file is often truncated. */
-  disk_car->CHS.cylinder=(disk_car->disk_size / disk_car->sector_size +
-      (uint64_t)disk_car->CHS.sector*(disk_car->CHS.head+1) - 1) /
-    disk_car->CHS.sector / (disk_car->CHS.head+1);
-  if(disk_car->CHS.cylinder > 0)
-    disk_car->CHS.cylinder--;
+  disk_car->geom.cylinders=(disk_car->disk_size / disk_car->sector_size +
+      (uint64_t)disk_car->geom.sectors_per_head * disk_car->geom.heads_per_cylinder - 1) /
+    disk_car->geom.sectors_per_head / disk_car->geom.heads_per_cylinder;
 }
 
 disk_t *file_test_availability(const char *device, const int verbose, const arch_fnct_t *arch, int testdisk_mode)
@@ -1284,7 +1287,7 @@ disk_t *file_test_availability(const char *device, const int verbose, const arch
         {
           try_readonly=1;
           close(hd_h);
-	  hd_h=-1;
+	  hd_h = -1;
         }
       }
 #endif
@@ -1339,7 +1342,7 @@ disk_t *file_test_availability(const char *device, const int verbose, const arch
     if(strncmp(device,"/dev/",5)!=0)
     {
 #if defined(HAVE_LIBEWF_H) && defined(HAVE_LIBEWF) && defined(HAVE_GLOB_H)
-      return fewf_init(device,verbose,arch,testdisk_mode);
+      return fewf_init(device, arch, testdisk_mode);
 #endif
     }
     return disk_car;
@@ -1354,8 +1357,8 @@ disk_t *file_test_availability(const char *device, const int verbose, const arch
   disk_car->data=data;
   disk_car->description=file_description;
   disk_car->description_short=file_description_short;
-  disk_car->read=file_read;
-  disk_car->write=((mode&O_RDWR)==O_RDWR?file_write:file_nowrite);
+  disk_car->pread=file_pread;
+  disk_car->pwrite=((mode&O_RDWR)==O_RDWR?file_pwrite:file_nopwrite);
   disk_car->sync=file_sync;
   disk_car->access_mode=((mode&O_RDWR)==O_RDWR?TESTDISK_O_RDWR:TESTDISK_O_RDONLY);
 #ifdef O_DIRECT
@@ -1376,7 +1379,7 @@ disk_t *file_test_availability(const char *device, const int verbose, const arch
     if(verbose>1)
       log_info("file_test_availability %s is a device\n", device);
     disk_car->sector_size=disk_get_sector_size(hd_h, device, verbose);
-    disk_get_geometry(&disk_car->CHS, hd_h, device, verbose);
+    disk_get_geometry(&disk_car->geom, hd_h, device, verbose);
     disk_car->disk_real_size=disk_get_size(hd_h, device, verbose, disk_car->sector_size);
 #ifdef BLKFLSBUF
     /* Little trick from Linux fdisk */
@@ -1388,6 +1391,7 @@ disk_t *file_test_availability(const char *device, const int verbose, const arch
     ioctl(hd_h, BLKFLSBUF);	/* ignore errors */
 #endif
     disk_get_model(hd_h, disk_car, verbose);
+    disk_get_hpa_dco(hd_h, disk_car);
   }
   else
 #endif
@@ -1405,10 +1409,10 @@ disk_t *file_test_availability(const char *device, const int verbose, const arch
     if(memcmp(buffer,"DOSEMU",6)==0 && *(unsigned long*)(buffer+11)>0)
     {
       log_info("%s DOSEMU\n",device);
-      disk_car->CHS.cylinder=*(unsigned long*)(buffer+15)-1;
-      disk_car->CHS.head=*(unsigned long*)(buffer+7)-1;
-      disk_car->CHS.sector=*(unsigned long*)(buffer+11);
-      disk_car->disk_real_size=(uint64_t)(disk_car->CHS.cylinder+1)*(disk_car->CHS.head+1)*disk_car->CHS.sector*disk_car->sector_size;
+      disk_car->geom.cylinders=*(unsigned long*)(buffer+15);
+      disk_car->geom.heads_per_cylinder=*(unsigned long*)(buffer+7);
+      disk_car->geom.sectors_per_head=*(unsigned long*)(buffer+11);
+      disk_car->disk_real_size=(uint64_t)disk_car->geom.cylinders * disk_car->geom.heads_per_cylinder * disk_car->geom.sectors_per_head * disk_car->sector_size;
       disk_car->offset=*(unsigned long*)(buffer+19);
     }
     else if(memcmp(buffer, evf_file_signature, 8)==0)
@@ -1420,16 +1424,16 @@ disk_t *file_test_availability(const char *device, const int verbose, const arch
       free(disk_car);
       close(hd_h);
 #if defined(HAVE_LIBEWF_H) && defined(HAVE_LIBEWF)
-      return fewf_init(device,verbose,arch,testdisk_mode);
+      return fewf_init(device, arch, testdisk_mode);
 #else
       return NULL;
 #endif
     }
     else
     {
-      disk_car->CHS.cylinder=0;
-      disk_car->CHS.head=255-1;
-      disk_car->CHS.sector=63;
+      disk_car->geom.cylinders=0;
+      disk_car->geom.heads_per_cylinder=255;
+      disk_car->geom.sectors_per_head=63;
       if((uint64_t)stat_rec.st_size > disk_car->offset)
       {
 	disk_car->disk_real_size=((uint64_t)stat_rec.st_size-disk_car->offset+disk_car->sector_size-1)/disk_car->sector_size*disk_car->sector_size;
@@ -1453,7 +1457,12 @@ disk_t *file_test_availability(const char *device, const int verbose, const arch
 #endif
   if(disk_car->disk_real_size!=0)
     return disk_car;
-  log_warning("Warning: can't get size for %s\n", device);
+  if(disk_car->model==NULL)
+    log_warning("Warning: can't get size for %s, sector size=%u\n",
+	disk_car->description(disk_car), disk_car->sector_size);
+  else
+    log_warning("Warning: can't get size for %s, sector size=%u - %s\n",
+	disk_car->description(disk_car), disk_car->sector_size, disk_car->model);
   free(data);
   free(disk_car->device);
   free(disk_car->model);
@@ -1465,12 +1474,10 @@ disk_t *file_test_availability(const char *device, const int verbose, const arch
 void hd_update_geometry(disk_t *disk_car, const int allow_partial_last_cylinder, const int verbose)
 {
   unsigned char *buffer;
-  uint64_t pos;
-  CHS_t pos_CHS;
   buffer=(unsigned char *)MALLOC(disk_car->sector_size);
   if(disk_car->autodetect!=0)
   {
-    if(disk_car->read(disk_car,disk_car->sector_size, buffer, 0)==0)
+    if((unsigned)disk_car->pread(disk_car, buffer, disk_car->sector_size, 0) == disk_car->sector_size)
     {
       if(verbose>1)
       {
@@ -1479,28 +1486,35 @@ void hd_update_geometry(disk_t *disk_car, const int allow_partial_last_cylinder,
       autoset_geometry(disk_car,buffer,1);
     }
   }
-  dup_CHS(&pos_CHS,&disk_car->CHS);
-  pos_CHS.cylinder++;
-  if(allow_partial_last_cylinder) {
-    pos_CHS.head=0;
-    pos_CHS.sector=1;
-  }
-  pos=CHS2offset(disk_car,&pos_CHS);
-#ifdef DJGPP
-  if(disk_car->description==disk_description)
+  if((unsigned int)(disk_car->geom.cylinders+1)!=0)	/* Avoid to wrap */
   {
-    struct info_disk_struct*data=disk_car->data;
-    data->CHSR.cylinder=0;
-  }
-#endif
-  if((unsigned int)(disk_car->CHS.cylinder+1)!=0)	/* Avoid to wrap */
-  {
-    if(disk_car->read(disk_car,disk_car->sector_size, buffer, pos)==0)
+    uint64_t pos;
+    CHS_t pos_CHS;
+    pos_CHS.cylinder=disk_car->geom.cylinders-1+1;
+    if(allow_partial_last_cylinder)
     {
-      disk_car->CHS.cylinder++;
-      if(disk_car->disk_size < (uint64_t)(disk_car->CHS.cylinder+1)*(disk_car->CHS.head+1)*disk_car->CHS.sector*disk_car->sector_size)
+      pos_CHS.head=0;
+      pos_CHS.sector=1;
+    }
+    else
+    {
+      pos_CHS.head=disk_car->geom.heads_per_cylinder-1;
+      pos_CHS.sector=disk_car->geom.sectors_per_head;
+    }
+    pos=CHS2offset(disk_car,&pos_CHS);
+#ifdef DJGPP
+    if(disk_car->description==disk_description)
+    {
+      struct info_disk_struct*data=disk_car->data;
+      data->geo_phys.cylinders=0;
+    }
+#endif
+    if((unsigned)disk_car->pread(disk_car, buffer, disk_car->sector_size, pos) == disk_car->sector_size)
+    {
+      disk_car->geom.cylinders++;
+      if(disk_car->disk_size < (uint64_t)disk_car->geom.cylinders * disk_car->geom.heads_per_cylinder * disk_car->geom.sectors_per_head * disk_car->sector_size)
       {
-	disk_car->disk_size=(uint64_t)(disk_car->CHS.cylinder+1)*(disk_car->CHS.head+1)*disk_car->CHS.sector*disk_car->sector_size;
+	disk_car->disk_size = (uint64_t)disk_car->geom.cylinders * disk_car->geom.heads_per_cylinder * disk_car->geom.sectors_per_head * disk_car->sector_size;
 	log_info("Computes LBA from CHS for %s\n",disk_car->description(disk_car));
       }
     }
@@ -1509,7 +1523,7 @@ void hd_update_geometry(disk_t *disk_car, const int allow_partial_last_cylinder,
   if(disk_car->description==disk_description)
   {
     struct info_disk_struct*data=disk_car->data;
-    data->CHSR.cylinder=disk_car->CHS.cylinder;
+    data->geo_phys.cylinders=disk_car->geom.cylinders;
   }
 #endif
   free(buffer);
@@ -1532,7 +1546,7 @@ void autoset_unit(disk_t *disk_car)
     return ;
   if(disk_car->arch==&arch_mac || 
       disk_car->arch==&arch_gpt || 
-      (disk_car->CHS.head==0 && disk_car->CHS.sector==1))
+      (disk_car->geom.heads_per_cylinder==1 && disk_car->geom.sectors_per_head==1))
     disk_car->unit=UNIT_SECTOR;
   else
     disk_car->unit=UNIT_CHS;
