@@ -25,6 +25,7 @@
 #include <config.h>
 #endif
  
+#include <stdio.h>
 #ifdef HAVE_STDLIB_H
 #include <stdlib.h>
 #endif
@@ -34,23 +35,23 @@
 #include <ctype.h>      /* tolower */
 #include "types.h"
 #include "common.h"
-#include "testdisk.h"
 #include "fnctdsk.h"
 #include "analyse.h"
 #include "lang.h"
 #include "intrf.h"
-#include "chgtype.h"
-#include "savehdr.h"
 #include "bfs.h"
 #include "bsd.h"
 #include "cramfs.h"
+#include "exfat.h"
 #include "ext2.h"
 #include "fat.h"
 #include "fatx.h"
 #include "hfs.h"
 #include "hfsp.h"
+#include "hpfs.h"
 #include "jfs_superblock.h"
 #include "jfs.h"
+#include "luks.h"
 #include "lvm.h"
 #include "md.h"
 #include "netware.h"
@@ -62,10 +63,9 @@
 #include "ufs.h"
 #include "xfs.h"
 #include "log.h"
-#include "partnone.h"
 
 static int check_part_none(disk_t *disk_car, const int verbose,partition_t *partition,const int saveheader);
-static int get_geometry_from_nonembr(const unsigned char *buffer, const int verbose, CHS_t *geometry);
+static int get_geometry_from_nonembr(const unsigned char *buffer, const int verbose, CHSgeometry_t *geometry);
 static list_part_t *read_part_none(disk_t *disk_car, const int verbose, const int saveheader);
 static list_part_t *init_part_order_none(const disk_t *disk_car, list_part_t *list_part);
 static void set_next_status_none(const disk_t *disk_car, partition_t *partition);
@@ -73,6 +73,9 @@ static int test_structure_none(list_part_t *list_part);
 static int is_part_known_none(const partition_t *partition);
 static void init_structure_none(const disk_t *disk_car,list_part_t *list_part, const int verbose);
 static const char *get_partition_typename_none_aux(const unsigned int part_type_none);
+static int set_part_type_none(partition_t *partition, unsigned int part_type);
+static unsigned int get_part_type_none(const partition_t *partition);
+static const char *get_partition_typename_none(const partition_t *partition);
 
 static const struct systypes none_sys_types[] = {
   {UP_BEOS,	"BeFS"},
@@ -81,6 +84,7 @@ static const struct systypes none_sys_types[] = {
   {UP_EXT3,	"ext3"},
   {UP_EXT4,	"ext4"},
 /*  {UP_EXTENDED,	"Extended"}, */
+  {UP_EXFAT,	"EXFAT"},
   {UP_FAT12,	"FAT12"},
   {UP_FAT16,	"FAT16"},
   {UP_FAT32,	"FAT32"},
@@ -128,24 +132,23 @@ arch_fnct_t arch_none=
   .get_geometry_from_mbr=get_geometry_from_nonembr,
   .check_part=check_part_none,
   .write_MBR_code=NULL,
-  .add_partition=NULL,
   .set_prev_status=set_next_status_none,
   .set_next_status=set_next_status_none,
   .test_structure=test_structure_none,
+  .get_part_type=get_part_type_none,
   .set_part_type=set_part_type_none,
-  .is_part_known=is_part_known_none,
   .init_structure=init_structure_none,
   .erase_list_part=NULL,
   .get_partition_typename=get_partition_typename_none,
-  .get_part_type=get_part_type_none
+  .is_part_known=is_part_known_none
 };
 
-unsigned int get_part_type_none(const partition_t *partition)
+static unsigned int get_part_type_none(const partition_t *partition)
 {
   return partition->upart_type;
 }
 
-int get_geometry_from_nonembr(const unsigned char *buffer, const int verbose, CHS_t *geometry)
+int get_geometry_from_nonembr(const unsigned char *buffer, const int verbose, CHSgeometry_t *geometry)
 {
   {
     /* Ugly hack to get geometry from FAT and NTFS */
@@ -155,8 +158,8 @@ int get_geometry_from_nonembr(const unsigned char *buffer, const int verbose, CH
       if(le16(fat_header->secs_track)>0 && le16(fat_header->secs_track)<=63 &&
           le16(fat_header->heads)>0 && le16(fat_header->heads)<=255)
       {
-        geometry->sector=le16(fat_header->secs_track);
-        geometry->head=le16(fat_header->heads)-1;
+        geometry->sectors_per_head=le16(fat_header->secs_track);
+        geometry->heads_per_cylinder=le16(fat_header->heads);
       }
     }
   }
@@ -221,7 +224,7 @@ static int test_structure_none(list_part_t *list_part)
   return 0;
 }
 
-int set_part_type_none(partition_t *partition, unsigned int part_type)
+static int set_part_type_none(partition_t *partition, unsigned int part_type)
 {
   partition->upart_type=(upart_type_t)part_type;
   return 0;
@@ -247,7 +250,7 @@ static int check_part_none(disk_t *disk_car,const int verbose,partition_t *parti
   switch(partition->upart_type)
   {
     case UP_BEOS:
-      ret=check_BeFS(disk_car,partition,verbose);
+      ret=check_BeFS(disk_car,partition);
       break;
     case UP_CRAMFS:
       ret=check_cramfs(disk_car,partition,verbose);
@@ -259,13 +262,16 @@ static int check_part_none(disk_t *disk_car,const int verbose,partition_t *parti
       break;
     case UP_EXTENDED:
       break;
+    case UP_EXFAT:
+      ret=check_EXFAT(disk_car, partition);
+      break;
     case UP_FAT12:
     case UP_FAT16:
     case UP_FAT32:
       ret=check_FAT(disk_car,partition,verbose);
       break;
     case UP_FATX:
-      ret=check_FATX(disk_car,partition,verbose);
+      ret=check_FATX(disk_car, partition);
       break;
     case UP_FREEBSD:
       ret=check_BSD(disk_car,partition,verbose,BSD_MAXPARTITIONS);
@@ -281,11 +287,14 @@ static int check_part_none(disk_t *disk_car,const int verbose,partition_t *parti
       ret=check_HPFS(disk_car,partition,verbose);
       break;
     case UP_JFS:
-      ret=check_JFS(disk_car,partition,verbose);
+      ret=check_JFS(disk_car, partition);
       break;
     case UP_LINSWAP:
     case UP_LINSWAP2:
-      ret=check_Linux_SWAP(disk_car,partition,verbose);
+      ret=check_Linux_SWAP(disk_car, partition);
+      break;
+    case UP_LUKS:
+    ret=check_LUKS(disk_car, partition);
       break;
     case UP_LVM:
       ret=check_LVM(disk_car,partition,verbose);
@@ -294,7 +303,7 @@ static int check_part_none(disk_t *disk_car,const int verbose,partition_t *parti
       ret=check_LVM2(disk_car,partition,verbose);
       break;
     case UP_NETWARE:
-      ret=check_netware(disk_car,partition,verbose);
+      ret=check_netware(disk_car, partition);
       break;
     case UP_NTFS:
       ret=check_NTFS(disk_car,partition,verbose,0);
